game = {}

--Game Variables
game_score = 0
game_time = 0

game_pause = false
pause_bg = love.graphics.newScreenshot()

game_tick = 0

--Game Images
fade = love.graphics.newImage("img/game/fade.png")

--BLOCKS

minDamage = 5
maxDamage = 10

block_tick = 0
block_int = 0.1

block_size = 16

block_fallSpeed = 300

blocks = {}

--COLORS
color_bg = {0, 100, 150}
color_dblock = {0, 0, 0, 255} -- damage block
color_sblock = {30, 200, 0, 255} --Score block
color_iblock = {0, 100, 170, 255} --invisibility block

color_movebar = {c = {255, 255, 255, 255}, d = {255, 255, 255, 255}} 


block_type = {
		{score = 0, damage = 5, pow = "none", pval = 0}, --basic
		{score = 4, damage = 0, pow = "none", pval = 0}, --score
		{score = 4, damage = 0, pow = "inv", pval = 8} --inv Powerup
	}
	
block_c = math.random(1, #block_type)

function addBlock(x, y, t)
	local sr = math.random(-10, 10)
	blocks[#blocks + 1] = {
			x = x,
			y = y,
			t = t,
			s = 100 + sr,
			
			damage = math.random(0, block_type[t].damage),
			score = math.random(0, block_type[t].score),
			pval =  math.random(0, block_type[t].pval),
			pow = block_type[t].pow
		}
end

function clearBlocks()
	for i=#blocks, 1, -1 do
		table.remove(blocks, i)
	end
end

function game.reset()
	clearBlocks()
	player.reset()
	
	game_score = 0
	game_time = 0
	game_tick = 0
end

function game.update(dt)
	if game_pause then
	
	else
		--GAME TICK LOOP
		game_tick = game_tick + dt
		if game_tick > 1 then
			game_time = game_time + 1
			game_tick = 0
		end
		
		--BLOCK TICK LOOP
		block_tick = block_tick + dt
		if block_tick > block_int then
			local r = math.random(0, 1000)
			local b = 0
			
			if r > 100 then
				b = 1
			elseif r > 10 and r < 100 then
				b = 2
			else
				b = 3
			end
			addBlock(math.random(0, screen.w - block_size), -block_size, b)
			block_tick = 0
		end
		
		--GAME LOOP
		
		--block update
		--LOOP
		for i,v in ipairs(blocks) do
			--Position update
			v.y = v.y + v.s * dt
			
			--Collision
			if checkCollision(player.x, player.y, player.w, player.h, v.x, v.y, block_size, block_size) then
				if player.inv then
					if v.t == 2 then
						game_score = game_score + v.score	
						table.remove(blocks, i)
					elseif v.t == 3 then
						if v.pow == "inv" then
							player.inv = true
							player.inv_time = player.inv_time + v.pval
						end
						table.remove(blocks, i)
					end
				else
					player.health = player.health - v.damage
					game_score = game_score + v.score
					if v.pow == "inv" then
						player.inv = true
						player.inv_time = player.inv_time + v.pval
					end
					table.remove(blocks, i)
				end
			end
			
		---Off screen remove
			if v.y > screen.h then
				table.remove(blocks, i)
			end
		end
		--COLOR UPDATE
		if player.inv then
			if color_dblock[4] > 130 then
				color_dblock[4] = color_dblock[4] - player.inv_fade * dt
			else
				player.col[4] = 130
			end
		else
			if color_dblock[4] <= 250 then
				color_dblock[4] = color_dblock[4] + player.inv_fade * dt
			else
				color_dblock[4] = 255
			end
		end
		
		
		--Power Update
		if player.inv then
			if player.inv_time >= 1 then
				player.inv_time = player.inv_time - dt
			else
				player.inv = false
				player.inv_time = 0
			end
		end
		player.update(dt)
	end
end

function game.draw()
	if game_pause then
		love.graphics.setPixelEffect(bw)
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.draw(pause_bg)
		love.graphics.setPixelEffect()
		
		love.graphics.setFont(fb)
		love.graphics.print("Paused", screen.c - (fb:getWidth("Paused") / 2), 32)
	else
		--BG
		love.graphics.draw(fade)

		
		
		love.graphics.setColor(color_movebar.c[1], color_movebar.c[2], color_movebar.c[3], color_movebar.c[4])
		love.graphics.rectangle("fill", 12, screen.h - 24, player.move_time, 12)
		love.graphics.setColor(0, 0, 0)
		love.graphics.rectangle("line", 12, screen.h - 24, player.move_maxTime, 12)

		
		--Blocks
		for i,v in ipairs(blocks) do
			if v.t == 1 then
				love.graphics.setColor(color_dblock[1], color_dblock[2], color_dblock[3], color_dblock[4])
			elseif v.t == 2 then
				love.graphics.setColor(color_sblock[1], color_sblock[2], color_sblock[3], color_sblock[4])
			elseif v.t == 3 then
				love.graphics.setColor(color_iblock[1], color_iblock[2], color_iblock[3], color_iblock[4])
			end
			love.graphics.rectangle("fill", v.x, v.y, block_size, block_size)
		end
		
		player.draw()
		
		--HUD
		love.graphics.setFont(f)
		love.graphics.setColor(255, 255, 255)
		love.graphics.print("Score: "..game_score.."\nTime: "..game_time.."\nHealth: "..player.health.."\nblocks : "..#blocks.."\nfps: "..love.timer.getFPS(), 12, 12)
	end
end

function game.keypressed(key)
	if key == "escape" then
		game_pause = not game_pause
		local s = love.graphics.newScreenshot()
		pause_bg = love.graphics.newImage(s)
	end
end











